@@ -1,21 +1,19 @@
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
+/* Bison interface for Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
-   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -28,38 +26,46 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_LIBCONFIG_YY_GRAMMAR_H_INCLUDED
+# define YY_LIBCONFIG_YY_GRAMMAR_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int libconfig_yydebug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     TOK_BOOLEAN = 258,
-     TOK_INTEGER = 259,
-     TOK_HEX = 260,
-     TOK_INTEGER64 = 261,
-     TOK_HEX64 = 262,
-     TOK_FLOAT = 263,
-     TOK_STRING = 264,
-     TOK_NAME = 265,
-     TOK_EQUALS = 266,
-     TOK_NEWLINE = 267,
-     TOK_ARRAY_START = 268,
-     TOK_ARRAY_END = 269,
-     TOK_LIST_START = 270,
-     TOK_LIST_END = 271,
-     TOK_COMMA = 272,
-     TOK_GROUP_START = 273,
-     TOK_GROUP_END = 274,
-     TOK_SEMICOLON = 275,
-     TOK_GARBAGE = 276,
-     TOK_ERROR = 277
-   };
+  enum yytokentype
+  {
+    TOK_BOOLEAN = 258,
+    TOK_INTEGER = 259,
+    TOK_HEX = 260,
+    TOK_INTEGER64 = 261,
+    TOK_HEX64 = 262,
+    TOK_FLOAT = 263,
+    TOK_STRING = 264,
+    TOK_NAME = 265,
+    TOK_EQUALS = 266,
+    TOK_NEWLINE = 267,
+    TOK_ARRAY_START = 268,
+    TOK_ARRAY_END = 269,
+    TOK_LIST_START = 270,
+    TOK_LIST_END = 271,
+    TOK_COMMA = 272,
+    TOK_GROUP_START = 273,
+    TOK_GROUP_END = 274,
+    TOK_SEMICOLON = 275,
+    TOK_GARBAGE = 276,
+    TOK_ERROR = 277
+  };
 #endif
 /* Tokens.  */
 #define TOK_BOOLEAN 258
@@ -83,31 +89,26 @@
 #define TOK_GARBAGE 276
 #define TOK_ERROR 277
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-
-/* Line 1676 of yacc.c  */
-#line 85 "grammar.y"
+#line 85 "grammar.y" /* yacc.c:1909  */
 
   int ival;
   long long llval;
   double fval;
   char *sval;
 
-
-
-/* Line 1676 of yacc.c  */
-#line 105 "grammar.h"
-} YYSTYPE;
+#line 105 "grammar.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
 
 
+int libconfig_yyparse (void *scanner, struct parse_context *ctx, struct scan_context *scan_ctx);
 
+#endif /* !YY_LIBCONFIG_YY_GRAMMAR_H_INCLUDED  */
