@@ -20,10 +20,14 @@
    ----------------------------------------------------------------------------
 */
 
+#include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <sys/stat.h>
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
 #include <libconfig.h>
 #include <tinytest.h>
 
@@ -48,7 +52,7 @@ static void parse_and_compare(const char *input_file, const char *output_file)
   remove("temp.cfg");
   TT_ASSERT_TRUE(config_write_file(&cfg, "temp.cfg"));
 
-  TT_ASSERT_FILE_EQ("temp.cfg", output_file);
+  TT_ASSERT_TXTFILE_EQ("temp.cfg", output_file);
   remove("temp.cfg");
 
   config_destroy(&cfg);
@@ -135,7 +139,7 @@ TT_TEST(ParsingAndFormatting)
     sprintf(output_file, "testdata/output_%d.cfg", i);
     printf("parsing %s\n", input_file);
 
-    if((access(input_file, F_OK) != 0) || (access(output_file, F_OK) != 0))
+    if(!tt_file_exists(input_file) || !tt_file_exists(output_file))
       break;
 
     parse_and_compare(input_file, output_file);
@@ -157,7 +161,7 @@ TT_TEST(ParseInvalidFiles)
     sprintf(input_file, "testdata/bad_input_%d.cfg", i);
     sprintf(error_file, "testdata/parse_error_%d.txt", i);
 
-    if((access(input_file, F_OK) != 0) || (access(error_file, F_OK) != 0))
+    if(!tt_file_exists(input_file) || !tt_file_exists(error_file))
       break;
 
     fp = fopen(error_file, "rt");
@@ -185,7 +189,7 @@ TT_TEST(ParseInvalidStrings)
     sprintf(input_file, "testdata/bad_input_%d.cfg", i);
     sprintf(error_file, "testdata/parse_error_%d.txt", i);
 
-    if((access(input_file, F_OK) != 0) || (access(error_file, F_OK) != 0))
+    if(!tt_file_exists(input_file) || !tt_file_exists(error_file))
       break;
 
     input_text = read_file_to_string(input_file);
